package de.dfki.madm.mlwizard;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.rapidminer.RapidMiner;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.OperatorCreationException;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.io.XrffExampleSource;
import com.rapidminer.operator.learner.bayes.NaiveBayes;
import com.rapidminer.operator.learner.functions.kernel.LibSVMLearner;
import com.rapidminer.operator.learner.functions.neuralnet.ImprovedNeuralNetLearner;
import com.rapidminer.operator.learner.lazy.KNNLearner;
import com.rapidminer.operator.learner.rules.RuleLearner;
import com.rapidminer.operator.learner.tree.DecisionTreeLearner;
import com.rapidminer.operator.learner.tree.RandomForestLearner;
import com.rapidminer.tools.OperatorService;
import com.rapidminer.tools.ParameterService;
import com.rapidminer.tools.XMLException;

/**
 * @brief administration of a knowledge base
 * some utility and convenience methods 
 */
public class Administrator {
    
    public static void main(String[] args) throws OperatorCreationException, OperatorException, FileNotFoundException, IOException, XMLException, InterruptedException {
        ParameterService.setParameterValue("rapidminer.home", "/home/remat/Projekte/PaREn/rapidminer");
        RapidMiner.setExecutionMode(RapidMiner.ExecutionMode.COMMAND_LINE);
        RapidMiner.init();
        
//        KnowledgeBase knowledgeBase = new KnowledgeBase();
        KnowledgeBase knowledgeBase = KnowledgeBase.read();
        
//        addDatasets("/ds/vanilla/meta-dataset/datasets", knowledgeBase, true);
//        Util.writeToRepository(knowledgeBase.getMetaFeatures(), "//scratch/knowledgebase/metafeatures");
        
        
//        ExampleSet A = c.getAccuracies();
//        ExampleSet B = knowledgeBase.getMetaFeatures();
//        for(Example a:A) {
//            System.out.print(a.getValue(A.getAttributes().get("accuracy")) + "      ");
//            Example b = Util.getExampleFromId(B, a.getNominalValue(A.getAttributes().getId()));
//            if(b != null ) {
//                System.out.println(b.getValue(b.getAttributes().get("naive_bayes")));
//            } else {
//                System.out.println("n.a.");
//            }
//        }
        
        
//        knowledgeBase.setMetaFeatures(Util.readFromRepository("//scratch/knowledgebase/metafeatures", ExampleSet.class));
//        knowledgeBase.write();
        
//        Administrator.initClassifiers(knowledgeBase);
        
//        for(Classifier classifier:knowledgeBase.getClassifiers()) {
//            threads.execute(new LearningThread(classifier, knowledgeBase.getMetaFeatures()));
//        }
        
//        List<String[]> numericalParams = new LinkedList<String[]>();
//        List<String[]> nominalParams = new LinkedList<String[]>();
//        numericalParams.add(new String[]{"SVM.gamma","[0.0001;16.0]"});
//        numericalParams.add(new String[]{"SVM.C","[0.0001;4096.0]"});
//        addClassifier(new Classifier("SVM2", Util.readFileAsString("/tmp/pre.xml"), Util.readFileAsString("/tmp/learn.xml"), numericalParams, nominalParams),
//                "/scratch/meta-dataset/SVM-parameters-ds.xrff", "/scratch/meta-dataset/SVM-accuracy-only.xrff", knowledgeBase);
        
        List<String[]> numericalParams = new LinkedList<String[]>();
        List<String[]> nominalParams = new LinkedList<String[]>();
        numericalParams.add(new String[]{"Random Forest."+RandomForestLearner.PARAMETER_NUMBER_OF_TREES,"[1;20]"});
        numericalParams.add(new String[]{"Random Forest."+RandomForestLearner.PARAMETER_MAXIMAL_DEPTH,"[1;20]"});
        numericalParams.add(new String[]{"Random Forest."+RandomForestLearner.PARAMETER_MINIMAL_GAIN,"[0.01;1.0]"});
        numericalParams.add(new String[]{"Random Forest."+RandomForestLearner.PARAMETER_MINIMAL_LEAF_SIZE,"[2;100]"});
        numericalParams.add(new String[]{"Random Forest."+RandomForestLearner.PARAMETER_MINIMAL_SIZE_FOR_SPLIT,"[2;100]"});
        numericalParams.add(new String[]{"Random Forest."+RandomForestLearner.PARAMETER_CONFIDENCE,"[0.001;0.5]"});
        Classifier c = new Classifier("Random Forest", RandomForestLearner.class, numericalParams, nominalParams);
        knowledgeBase.addClassifier(c, true);
        knowledgeBase.awaitTermination(1, TimeUnit.DAYS);
        knowledgeBase.write();
    }
    
    public static void initClassifiers(KnowledgeBase knowledgeBase) throws IOException, XMLException, OperatorException, OperatorCreationException {
        
        List<String[]> numericalParams = new LinkedList<String[]>();
        List<String[]> nominalParams = new LinkedList<String[]>();
        
        numericalParams.clear();
        numericalParams.add(new String[]{"SVM."+LibSVMLearner.PARAMETER_GAMMA,"[0.0001;16.0]"});
        numericalParams.add(new String[]{"SVM."+LibSVMLearner.PARAMETER_C,"[0.0001;4096.0]"});
        nominalParams.clear();
        addClassifier(new Classifier("SVM", LibSVMLearner.class, numericalParams, nominalParams),
                "/scratch/meta-dataset/SVM-parameters-ds.xrff", "/scratch/meta-dataset/SVM-accuracy-only.xrff", knowledgeBase);

        numericalParams.clear();
        nominalParams.clear();
        nominalParams.add(new String[]{"Naive Bayes."+NaiveBayes.PARAMETER_LAPLACE_CORRECTION,"true,false"});
        addClassifier(new Classifier("Naive Bayes", NaiveBayes.class, numericalParams, nominalParams),
                "/scratch/meta-dataset/Naive Bayes-parameters-ds.xrff", "/scratch/meta-dataset/Naive Bayes-accuracy-only.xrff", knowledgeBase);
        
        numericalParams.clear();
        numericalParams.add(new String[]{"Neural Net."+ImprovedNeuralNetLearner.PARAMETER_LEARNING_RATE,"[0.01;1.0]"});
        nominalParams.clear();
        nominalParams.add(new String[]{"Neural Net.decay","true,false"});
        addClassifier(new Classifier("Neural Net", ImprovedNeuralNetLearner.class, numericalParams, nominalParams),
                "/scratch/meta-dataset/Neural Net-parameters-ds.xrff", "/scratch/meta-dataset/Neural Net-accuracy-only.xrff", knowledgeBase);
        
        numericalParams.clear();
        numericalParams.add(new String[]{"Decision Tree."+DecisionTreeLearner.PARAMETER_MINIMAL_SIZE_FOR_SPLIT, "[2.0;100.0]"});
        numericalParams.add(new String[]{"Decision Tree."+DecisionTreeLearner.PARAMETER_MINIMAL_LEAF_SIZE, "[1.0;100.0]"});
        numericalParams.add(new String[]{"Decision Tree."+DecisionTreeLearner.PARAMETER_MINIMAL_GAIN, "[0.05;5.0]"});
        numericalParams.add(new String[]{"Decision Tree."+DecisionTreeLearner.PARAMETER_MAXIMAL_DEPTH, "[5.0;100.0]"});
        numericalParams.add(new String[]{"Decision Tree."+DecisionTreeLearner.PARAMETER_CONFIDENCE, "[0.05;0.5]"});
        nominalParams.clear();
        addClassifier(new Classifier("Decision Tree", DecisionTreeLearner.class, numericalParams, nominalParams),
                "/scratch/meta-dataset/Decision Tree-parameters-ds.xrff", "/scratch/meta-dataset/Decision Tree-accuracy-only.xrff", knowledgeBase);
        
        numericalParams.clear();
        numericalParams.add(new String[]{"k-NN."+KNNLearner.PARAMETER_K,"[1;1000]"});
        nominalParams.clear();
        nominalParams.add(new String[]{"k-NN."+KNNLearner.PARAMETER_WEIGHTED_VOTE,"true,false"});
        addClassifier(new Classifier("k-NN", KNNLearner.class, numericalParams, nominalParams),
                "/scratch/meta-dataset/k-NN-parameters-ds.xrff", "/scratch/meta-dataset/k-NN-accuracy-only.xrff", knowledgeBase);
        
        numericalParams.clear();
        numericalParams.add(new String[]{"Rule Induction.sample_ratio","[0.1;1.0]"});
        numericalParams.add(new String[]{"Rule Induction.minimal_prune_benefit","[0.1;1.0]"});
        numericalParams.add(new String[]{"Rule Induction.pureness","[0.1;0.9]"});
        nominalParams.clear();
        nominalParams.add(new String[]{"Rule Induction.criterion","information_gain,accuracy"});
        addClassifier(new Classifier("Rule Induction", RuleLearner.class, numericalParams, nominalParams),
                "/scratch/meta-dataset/Rule Induction-parameters-ds.xrff", "/scratch/meta-dataset/Rule Induction-accuracy-only.xrff", knowledgeBase);
        
    }
    
   
    /**
     * adds a classifier with pre-computed accuracies and parameters 
     * @param classifier the classifier
     * @param parametersFile the file containing the optimal parameter values
     * @param accuraciesFile the file containing the accuracy values
     * @param knowledgeBase the knowledge base
     * @throws OperatorCreationException
     * @throws OperatorException
     * @throws IOException
     * @throws XMLException
     */
    public static void addClassifier(Classifier classifier, String parametersFile, String accuraciesFile, KnowledgeBase knowledgeBase)
            throws OperatorCreationException, OperatorException, IOException, XMLException {
        classifier.setParameters(parametersFile);
        classifier.setAccuracies(accuraciesFile);
        knowledgeBase.addClassifier(classifier, false);
    }
   
    
    /**
     * adds a complete folder of datasets (not recursively) in the XRFF format to the knowledge base. Files must end with .xrff or .XRFF. 
     * @param folder the folder of the datasets
     * @param knowledgeBase the knowledge base the datasets should be added to
     * @throws OperatorCreationException
     * @throws OperatorException
     * @throws FileNotFoundException
     * @throws IOException
     * @throws InterruptedException 
     * @throws XMLException 
     */
    public static void addDatasets(String folder, KnowledgeBase knowledgeBase, boolean store) throws OperatorCreationException, OperatorException, FileNotFoundException, IOException, XMLException, InterruptedException {
        XrffExampleSource reader = OperatorService.createOperator(XrffExampleSource.class);
        for(File file:new File(folder).listFiles()) {
            if (file.isFile() && (file.getName().endsWith(".xrff") || file.getName().endsWith(".XRFF"))) {
                reader.setParameter(XrffExampleSource.PARAMETER_DATA_FILE, file.getAbsolutePath());
                String name = file.getName().substring(0, file.getName().length() - 5);
                ExampleSet e = reader.read();
//                if(e.size() < 1000 && e.getAttributes().size() < 10) {
                    Logger.getAnonymousLogger().log(Level.INFO, name);
                    knowledgeBase.addDataset(e, name, store);
//                }
            }
        }
    }
}
