/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.mlwizard.gui;

import java.util.HashMap;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import com.rapidminer.example.Attribute;
import com.rapidminer.tools.Tools;
import com.rapidminer.tools.container.Pair;

import de.dfki.madm.mlwizard.Classifier;
import de.dfki.madm.mlwizard.functionality.Regressioner.RegressionResult;

/**
 * @author Matthias Reif
 */
public class RegressionTableModel extends AbstractTableModel {

	private static final long serialVersionUID = -4350774461195655604L;
	private String[] columnNames = { "Classifier", "Predicted Accuracy", "Root Mean Squared Error", "Evaluate" };
    Attribute classifierAttribute, predictionAttribute, rmseAttribute;
    private HashMap<String, Boolean> evaluate;
    private RegressionResult result;

    public RegressionTableModel(RegressionResult result) {
        this.result = result;
        evaluate = new HashMap<String, Boolean>();
        String bestClassifier = "";
        double highestPrediction = 0.0;
        for(Pair<Classifier, Double> entry:result.predictions) {
            evaluate.put(entry.getFirst().getName(), Boolean.FALSE);
            if(entry.getSecond() > highestPrediction) {
                highestPrediction = entry.getSecond();
                bestClassifier = entry.getFirst().getName();
            }
        }
        evaluate.put(bestClassifier, Boolean.TRUE);
    }

    public RegressionResult getRegressionResult() {
        return result;
    }

    public HashMap<String, Boolean> getEvaluate() {
        return evaluate;
    }
    
    public Vector<String> getIfEvaluate() {
        Vector<String> result = new Vector<String>();
        for(String classifierName:evaluate.keySet()) {
            if(evaluate.get(classifierName)) {
                result.add(classifierName);
            }
        }
        return result;
    }

    

    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    @Override
    public int getRowCount() {
        return result.predictions.size();
    }

    @Override
    public String getColumnName(int col) {
        return columnNames[col];
    }

    @Override
    public Object getValueAt(int row, int col) {
        Pair<Classifier, Double> prediction = result.predictions.get(row);
        String classifierName = prediction.getFirst().getName();
        switch(col) {
        case 0:
            return classifierName;
        case 1:
            // -- Values >1 may happen because of regression modeling, but looks strange to the user --
            return Tools.formatNumber(Math.min(prediction.getSecond(), 1.0));
        case 2:
            return Tools.formatNumber(prediction.getFirst().getRMSE());
        case 3:
            return evaluate.get(classifierName);
        }
        return null;
    }

    public Class <? extends Object> getColumnClass(int col) {
        if(col == 3) {
            return Boolean.class;
        } else if(col == 4) {
            return Object.class;
        } else {
            return String.class;
        }
    }

    public boolean isCellEditable(int row, int col) {
        return col == 3;
    }

    public void setValueAt(Object value, int row, int col) {
        if(col == 3) {
            evaluate.put(result.predictions.get(row).getFirst().getName(), (Boolean)value);
        }
        fireTableCellUpdated(row, col);
    }

}
