/*
 *  RapidMiner PaREn Automatic System Construction Extension 
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.dfki.madm.mlwizard.landmarking;

import java.util.Map;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.OperatorCreationException;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.learner.lazy.KNNLearner;
import com.rapidminer.operator.validation.XValidation;
import com.rapidminer.tools.OperatorService;

/**
 * This landmark learner uses the {@link KNNLearner}, where the k-value is by
 * default 1 except it is changed by the user. The landmark values of the
 * learner are determined using {@link XValidation}.
 * 
 * @author Sarah Daniel
 * 
 */
public class KNNLearnerLm extends AbstractLandmarker {

    public KNNLearnerLm(ExampleSet exampleSet, Map<String, String> parameters) throws OperatorCreationException, OperatorException {
        // default value of k = 1

            this.name = parameters.get(LandmarkingParameters.PARAMETER_KNN_K_VALUE) + LandmarkingParameters.PARAMETER_KNN_LM;

            KNNLearner learner = OperatorService.createOperator(KNNLearner.class);
            learner.setParameter(KNNLearner.PARAMETER_K, parameters.get(LandmarkingParameters.PARAMETER_KNN_K_VALUE));
            exampleSet = normalizeExampleSet(exampleSet, parameters);

            xValidation(exampleSet, learner, parameters);

    }

}
