package de.dfki.madm.mlwizard.optimization;

import java.util.Iterator;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.meta.ESParameterOptimization;
import com.rapidminer.operator.meta.EvolutionaryParameterOptimizationOperator;
import com.rapidminer.parameter.UndefinedParameterError;
import com.rapidminer.tools.RandomGenerator;
import com.rapidminer.tools.math.optimization.ec.es.ESOptimization;

/**
 * override of the GA that uses a faked random generator to introduce defined start points
 * @author remat
 */
public class ESOptimizationPaREn extends EvolutionaryParameterOptimizationOperator {

    
    /**
     * all values of individual one, then all values of individuals to, ...
     */
    private Iterable<Double> values;
    
    
    public ESOptimizationPaREn(OperatorDescription description) {
        super(description);
        values = null;
    }

    public void setStartValues(Iterable<Double> values) {
        this.values = values;
    }
    
    @Override
    protected ESOptimization createOptimizer(RandomGenerator random) throws UndefinedParameterError {
        return new ESParameterOptimization(
                this, 
                getOptimizationOperators().length, 
                ESOptimization.INIT_TYPE_RANDOM, 
                getParameterAsInt(PARAMETER_MAX_GENERATIONS), 
                getParameterAsInt(PARAMETER_GENERATIONS_WITHOUT_IMPROVAL),
                getParameterAsInt(PARAMETER_POPULATION_SIZE), 
                getParameterAsInt(PARAMETER_SELECTION_TYPE), 
                getParameterAsDouble(PARAMETER_TOURNAMENT_FRACTION), 
                getParameterAsBoolean(PARAMETER_KEEP_BEST), 
                getParameterAsInt(PARAMETER_MUTATION_TYPE), 
                getParameterAsDouble(PARAMETER_CROSSOVER_PROB),
                getParameterAsBoolean(PARAMETER_SHOW_CONVERGENCE_PLOT), 
                new StartPopulationGenerator(),
                this);
    }

    class StartPopulationGenerator extends RandomGenerator {
        
        private static final long serialVersionUID = 7358360862155716956L;

        private Iterator<Double> valuesIt = null;
        
        public StartPopulationGenerator() {
            super(0);
            if(values != null) {
                valuesIt = values.iterator();
            } else {
                getLog().logWarning("No start vaules defined for start population generator!");
            }

        }
        
        @Override
        public double nextDoubleInRange(double lowerBound, double upperBound) {
            if(valuesIt != null && valuesIt.hasNext()) {
                return valuesIt.next();
            } else {
                return super.nextDoubleInRange(lowerBound, upperBound);
            }
        }
    }

}
